<?
/***************************************************************************

Browser Emulating fopen() functions v1.0
(c) Kai Blankenhorn
www.bitfolge.de/en
kaib@bitfolge.de


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************


Changelog:

v1.5
	added Basic HTTP user authorization
	minor optimizations

v1.0
	initial release



***************************************************************************/

/**
 * Browser Emulating fopen() function
 * Opens a remote file, pretending to be a web browser by sending appropriate header fields.
 * This is useful for sites which prevent script from reading their pages by checking for certain fields.
 * Returns a file handle which can be used with the usual file handling functions, or FALSE if there
 * was an error (the file doesn't exist or you have no access rights).
 * 
 * Usage:
 * function be_fopen($url, $additionalHeaders=Array(), $PostData=Array(), $authUser="", $authPass="")
 * 
 * $url                  The page to be opened.
 *                       Valid schemes for $url are:
 *                       http://www.server.com
 *                       http://www.server.com/any/path/and/file?param=val
 *                       www.server.com
 * 
 * $additionalHeaders	 Associative array containing additional header fields to be sent along the request
 *                       optional
 *                       Example:    Array("Referer"=>"http://previous.server.com");
 * 
 * $PostData             Associative array containing any necessary POST data. If this is empty, the GET method
 *                       is used. Otherwise, POST is used.
 *                       optional
 *                       Example:    Array("submit"=>"OK","id"=>"123456789")
 * 
 * $authUser             User name required to access $url
 *                       optional
 *
 * $authPass             The password for $authUser
 *                       optional
 * 
 **/

function be_fopen($url, $additionalHeaders=Array(), $PostData=Array(), $authUser="", $authPass="") {
	$debug = false;

	preg_match("~([a-z]*://)?([^/]*)(/.*)?~i", $url, $matches);
	$protocol = $matches[1];
	$server = $matches[2];
	$path = $matches[3];
	if ($path=="") $path = "/";
	$socket = false;
	$socket = fsockopen($server, 80);
	if ($socket) {
		$headers["Host"] = $server;
		
		/*******************************************************************************/
		/**************    YOU MAX SET THE USER AGENT STRING HERE    *******************/
		/*                                                                             */
		/* default is "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)",            */
		/* which means Internet Explorer 6.0 on Win2000                                */
		
		$headers["User-Agent"] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)";
		
		/*******************************************************************************/
		
		$headers = array_merge($headers, $additionalHeaders);
		if ($authUser!="" AND $authPass!="") {
			$headers["Authorization"] = "Basic ".base64_encode($authUser.":".$authPass);
		}

		if (count($PostData)==0) {
			$request = "GET $path HTTP/1.0\r\n";
		} else {
			$request = "POST $path HTTP/1.0\r\n";
		}
		
		if ($debug) echo $request;
	    fputs ($socket, $request);

		if (count($PostData)>0) {
			$PostStringArray = Array();
			foreach ($PostData AS $key=>$value) {
				$PostStringArray[] = "$key=$value";
			}
			$PostString = join("&", $PostStringArray);
			$headers["Content-Length"] = strlen($PostString);
		}
		
		foreach ($headers AS $key=>$value) {
			if ($debug) echo "$key: $value\n";
		    fputs($socket, "$key: $value\r\n");
		}
		if ($debug) echo "\n";
		fputs($socket, "\r\n");
		if (count($PostData)>0) {
			if ($debug) echo "$PostString";
			fputs($socket, $PostString."\r\n");
		}
	}
	if ($debug) echo "\n";
	if ($socket) {
		$line = fgets($socket, 1000);
		if ($debug) echo "$line";
		$status = substr($line,9,3);
		do {
			$line = fgets($socket, 1000);
			if ($debug) echo "$line";
			if ($status=="401" AND strpos($line,"WWW-Authenticate: Basic realm=\"")===0) {
				fclose($socket);
				return FALSE;
			}
		} while (trim($line)!="");
	}
	return $socket;
}

/**
 * Browser Emulating file() function
 * Opens a remote file, pretending to be a web browser by sending appropriate header fields.
 * This is useful for sites which prevent script from reading their pages by checking for certain fields.
 * Returns the file in an array, each element containing one line (without trailing line break), or FALSE
 * if there was an error (the file doesn't exist or you have no access rights).
 * 
 * Usage:
 * function be_file($url, $additionalHeaders=Array(), $PostData=Array(), $authUser="", $authPass="")
 * 
 * $url                  The page to be opened.
 *                       Valid schemes for $url are:
 *                       http://www.server.com
 *                       http://www.server.com/any/path/and/file?param=val
 *                       www.server.com
 * 
 * $additionalHeaders	 Associative array containing additional header fields to be sent along the request
 *                       optional
 *                       Example:    Array("Referer"=>"http://previous.server.com");
 * 
 * $PostData             Associative array containing any necessary POST data. If this is empty, the GET method
 *                       is used. Otherwise, POST is used.
 *                       optional
 *                       Example:    Array("submit"=>"OK","id"=>"123456789")
 * 
 * $authUser             User name required to access $url
 *                       optional
 *
 * $authPass             The password for $authUser
 *                       optional
 * 
 **/
 
function be_file($url, $additionalHeaders=Array(), $PostData=Array(), $authUser="", $authPass="") {
	$file = Array();
	$socket = be_fopen($url, $additionalHeaders, $PostData, $authUser="", $authPass="");
	if ($socket) {
		$file = Array();
		while (!feof($socket)) {
			$file[] = fgets($socket, 10000);
		}
	} else {
		return FALSE;
	}
	return $file;
}

?>